# -*- coding: utf-8 -*-
# Elisa - Home multimedia server
# Copyright (C) 2006-2008 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 3.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Elisa with Fluendo's plugins.
#
# The GPL part of Elisa is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Elisa" in the root directory of this distribution package
# for details on that license.
#
# Author: Olivier Tilloy <olivier@fluendo.com>

from elisa.core.components.model import Model
from elisa.core.media_uri import MediaUri
from elisa.core.application import PICTURES_CACHE
from elisa.core.utils import defer

from elisa.plugins.base.models.image import ImageModel

from distutils.version import LooseVersion

from twisted.web.client import getPage

import os

try:
    from hashlib import md5
except ImportError:
    # hashlib is new in Python 2.5
    from md5 import md5


class PluginModel(Model):

    """
    A model that represents a plugin and its metadata as understood by Elisa.

    @note: this model may be enriched along the way with new metadata

    @ivar name:         the name of the plugin
    @type name:         C{unicode}
    @ivar version:      the version number of the plugin
    @type version:      L{distutils.version.LooseVersion}
    @ivar author_name:  the name of the author of the plugin
    @type author_name:  C{unicode}
    @ivar author_email: the e-mail address of the author of the plugin
    @type author_email: C{unicode}
    @ivar platforms:    a list of compatible platforms (typical values are:
                        C{linux}, C{win32}, C{macosx})
    @type platforms:    C{list} of C{unicode}
    @ivar license:      the license of the plugin
    @type license:      C{unicode}
    @ivar website:      the home page of the plugin
    @type website:      C{unicode}
    @ivar category:     a unique category the plugin belongs to
    @type category:     C{unicode}
    @ivar tags:         a list of tags that define the plugin
    @type tags:         C{list} of C{unicode}
    @ivar egg_name:     the egg name
    @type egg_name:     C{unicode}
    @ivar uri:          the URI where the egg can be downloaded from
    @type uri:          C{unicode}
    @ivar summary:      a short text describing the plugin
    @type summary:      C{unicode}
    @ivar description:  a detailed description of what the plugin does
    @type description:  C{unicode}
    @ivar icons:        a list of icons for the plugin
    @type icons:        C{list} of
                        L{elisa.plugins.base.models.image.ImageModel}
    @ivar screenshots:  a list of screenshot images for the plugin
    @type screenshots:  C{list} of
                        L{elisa.plugins.base.models.image.ImageModel}

    @ivar enabled:      whether the plugin is currently enabled (this attribute
                        is not part of the plugin metadata)
    @type enabled:      C{bool}
    """

    def __init__(self):
        super(PluginModel, self).__init__()
        self.name = ''
        self.version = LooseVersion()
        self.author_name = ''
        self.author_email = ''
        self.platforms = []
        self.license = ''
        self.website = ''
        self.category = ''
        self.tags = []
        self.egg_name = ''
        self.uri = ''
        self.summary = ''
        self.description = ''
        self.icons = []
        self.screenshots = []
        self.enabled = False

    @classmethod
    def from_dict(cls, dictionary):
        """
        Class method that instantiates and returns a plugin model from a
        dictionary as provided by the plugin repository.

        @param dictionary: a dictionary as provided by the plugin repository
        @type dictionary:  C{dict}

        @return:           a plugin model populated accordingly
        @rtype:            L{PluginModel}
        """
        plugin = cls()
        for key, value in dictionary.iteritems():
            if key in ('name', 'author_name', 'author_email', 'license',
                       'website', 'category', 'tags', 'egg_name', 'uri',
                       'summary', 'description'):
                setattr(plugin, key, value)
            elif key == 'version':
                plugin.version.parse(value)
            elif key == 'platforms':
                # Platforms is a comma-separated list of platforms serialized
                # as a string.
                platforms = value.split(',')
                for platform in platforms:
                    platform = platform.strip()
                    if platform != 'UNKNOWN':
                        plugin.platforms.append(platform)
            elif key == 'icons':
                icons = eval(value)
                for icon in icons:
                    model = ImageModel()
                    model.references.append(MediaUri(icon['uri']))
                    plugin.icons.append(model)
            elif key == 'screenshots':
                screenshots = eval(value)
                for screenshot in screenshots:
                    model = ImageModel()
                    model.references.append(MediaUri(screenshot['uri']))
                    plugin.screenshots.append(model)
        return plugin

    @classmethod
    def from_distribution(cls, distribution):
        """
        Class method that instantiates and returns a plugin model from a
        distribution as provided by L{pkg_resources}.

        @param distribution: a distribution as provided by L{pkg_resources}
        @type distribution:  L{pkg_resources.Distribution}

        @return:             a plugin model populated accordingly
        @rtype:              L{PluginModel}
        """
        plugin = cls()
        plugin.name = distribution.project_name
        plugin.version.parse(distribution.version)
        plugin.author_name = distribution.author
        plugin.license = distribution.license
        plugin.egg_name = distribution.egg_name()
        plugin.summary = distribution.summary
        plugin.description = distribution.description
        return plugin

    def printable_name(self):
        """
        Return a human-friendly name for the plugin.

        @rtype: C{unicode}
        """
        prefix = 'elisa-plugin-'
        name = self.name
        if name.startswith(prefix):
            name = name[len(prefix):]
        name = name.capitalize()
        return name

    def cache_image(self, uri):
        """
        Return the path to the image cached on disk corresponding to the given
        URI.

        If the image is not cached yet, download it and cache it.

        @param uri: the URI to the image
        @type uri:  L{elisa.core.media_uri.MediaUri}

        @return:    the path to the cache file and a deferred triggered when
                    the image is cached
        @rtype:     (C{str}, L{elisa.core.utils.defer.Deferred})
        """
        # TODO: this code should be factorized in a common caching module.
        if not os.path.exists(PICTURES_CACHE):
            os.makedirs(PICTURES_CACHE, 0755)
        cache_file = '%s.%s' % (md5(str(uri)).hexdigest(), uri.extension)
        thumbnail_file = os.path.join(PICTURES_CACHE, cache_file)
        if os.path.exists(thumbnail_file):
            return (thumbnail_file, defer.succeed(thumbnail_file))
        else:
            # Download the thumbnail and cache it
            def got_thumbnail(data):
                fd = file(thumbnail_file, 'wb')
                fd.write(data)
                fd.close()
                return thumbnail_file

            dfr = getPage(str(uri))
            dfr.addCallback(got_thumbnail)
            return (thumbnail_file, dfr)
